# Copyright Impulse Accelerated Technologies, Inc. and/or its
# licensors. All rights reserved.

# This Tcl script is intended to be run using the Quartus II supplied tcl interpreter tclsh located at $QUARTUS_ROOTDIR/bin
# Execute at the command line as:
# 		Windows: %QUARTUS_ROOTDIR%\bin\tclsh gen_vhdl_sim.tcl
# 		Linux: $QUARTUS_ROOTDIR/bin/tclsh gen_vhdl_sim.tcl

# NOTES:
# 	Environement variable QUARTUS_ROOTDIR must be set to Quartus installation directory which typically looks like:
#		Windows: C:\altera\12.1sp1\quartus
#		Linux: /opt/altera/12.1sp1/quartus
#  Script will look for "../../*_ip.vhd" to scan for comments that include complete command line information for
#	 invoking the MegaWizard via $QUARTUS_ROOTDIR/bin/qmegawiz

# check that QUARTUS_ROOTDIR is set
if {[info exists ::env(QUARTUS_ROOTDIR)]} {
	puts "QUARTUS_ROOTDIR is set to $::env(QUARTUS_ROOTDIR)"
} else {
	puts "!!! ERROR: Environment variable QUARTUS_ROOTDIR must be defined"
	exit -1;
}

# bin directory is either "bin", "bin64", or "bin32", find one

set QUARTUS_BINDIR [ file join $::env(QUARTUS_ROOTDIR) bin64]
set foundDir [file exists $QUARTUS_BINDIR] 
if {$foundDir == 1 } {
	puts "QUARTUS_BINDIR is set to $QUARTUS_BINDIR"
} else {
	puts "Quartus binary directory is not at $QUARTUS_BINDIR"
	set QUARTUS_BINDIR [ file join $::env(QUARTUS_ROOTDIR) bin32]
	set foundDir [file exists $QUARTUS_BINDIR] 
	if {$foundDir == 1 } {
		puts "QUARTUS_BINDIR is set to $QUARTUS_BINDIR"
	} else {
		puts "Quartus binary directory is not at $QUARTUS_BINDIR"
		set QUARTUS_BINDIR [ file join $::env(QUARTUS_ROOTDIR) bin]
		set foundDir [file exists $QUARTUS_BINDIR] 
		if {$foundDir == 1 } {
			puts "QUARTUS_BINDIR is set to $QUARTUS_BINDIR"
		} else {
			puts "Quartus binary directory is not at $QUARTUS_BINDIR"
			puts "!!! ERROR: Could not find Quartus binary directory"
			exit -1;
		}
	}
}


# check that qmegawiz is where it is expected
set qmegawizBin [file join $QUARTUS_BINDIR qmegawiz]
if {[file exists $qmegawizBin]} {
	puts "qmegawiz found at $qmegawizBin"
} else {
	# might be Windows, try again with suffix ".exe"
	append qmegawizBin ".exe"
	if {[file exists $qmegawizBin]} {
		puts "qmegawiz found at $qmegawizBin"
	} else {
   	puts "!!! ERROR: qmegawizBin was NOT found at $qmegawizBin"
   	puts "!!! ERROR: Environment variable QUARTUS_ROOTDIR must be set correctly to find qmegawiz executable"
   	exit -1;
	}
}

# find ../*_ip.vhd, within its comments will be the command line for invoking MegaWizard qmegawiz
set searchFor [file join ".." ".." "*_ip.vhd"]
set ipVhdlFiles [glob -nocomplain -type f $searchFor]

if { [llength $ipVhdlFiles] == 0 } {
	puts "!!! ERROR: No files to search found for pattern $searchFor, exiting."
	exit -1;
}
puts "Found these '_ip.vhd files to search: $ipVhdlFiles"

set tmpDir "tmp"

puts "remove old $tmpDir directory"
file delete -force $tmpDir
puts "make new $tmpDir directory"
file mkdir $tmpDir

foreach ipFile $ipVhdlFiles {
	puts "Searching file $ipFile for qmegawiz commands"
	set fin [open $ipFile r]
	set lineNum 0
	while {[gets $fin line] >= 0} {
		incr lineNum
		if {[regexp {\-\- qmegawiz(.*$)} $line whole mwArgs]} {
			puts "Found qmegawiz command arguments at line #$lineNum:\n$mwArgs"
			
			# if .vho file exists, don't run qmegawiz (names are unique to static parameters and need only be created once)
			set simBaseFilePos [lsearch $mwArgs "*.vhd"]
			if { $simBaseFilePos < 0 } {
				puts "!!! ERROR: Could not find simulationg file *.vhd in qmegawiz arguments $mwArgs, exiting."
				exit -1;
			}
			set simFileNameBase [lindex [split [lindex $mwArgs $simBaseFilePos] "."] 0]
			set simFileName "$simFileNameBase.vho"
			set simFileNameVhd "$simFileNameBase.vhd"
			puts "Simulation file to be created $simFileName"
			if {[file exists $simFileName] || [file exists $simFileNameVhd]} {
				puts "Simulation file $simFileNameBase.vho/.vhd exists, skipping call to qmegawiz (delete .vho/.vhd file to force recreation)"
			} else {
				puts "Creating simulation file $simFileName"
   			set qmegawizCmd "$qmegawizBin $mwArgs"
   			
   			puts "cd into $tmpDir directory"
   			cd $tmpDir
   
   			puts "cleanup old $tmpDir directory"
   			foreach f [glob -nocomplain "*"] {
   				file delete -force $f
   			}
   
   			puts "Executing qmegawiz:"
   			if { [catch [ eval exec $qmegawizCmd] ] } {
   				puts "!!! ERROR:Executing qmegawiz failed for command:\n$qmegawizCmd\n"
   				exit -1
   			}
   			
   			puts "copying .vho and .hex files needed for simulation up to parent directory"
				set numCopied 0
   			foreach f [glob -nocomplain "*.vho" "*.hex" "*.vhd"] {
   				file copy -force $f ".."
					incr numCopied
   			}
				if { $numCopied < 1 } {
					puts "No .vho or .hex files were generated, instead copying any .vhd files to parent directory"
					foreach f [glob -nocomplain "*.vhd"] {
						file copy -force $f ".."
						incr numCopied
					}
				}
				if { $numCopied < 1 } {
					puts "!!! No .vho, .hex, nor .vhd files were copied to parent directory from command:\n$qmegawizCmd\n"
					exit -1
				}
   			
   			puts "cd into parent directory"
   			cd ".."
			}
		}
	}
	puts "Done searching file $ipFile for qmegawiz commands"
	close $fin
}

puts "final remove old $tmpDir directory"
file delete -force $tmpDir


# invoke megawizard for each found, need the resulting .vho and any .hex files for HDL simulation 

puts "Done - SUCCESS!"

