library ieee;
use ieee.std_logic_1164.all;

package altera_float is
    component latency_delay is
        generic (
            WIDTH : Integer := 1;
            LATENCY : Integer := 1
        );
        port (
            clk : in std_ulogic;
            a : in std_ulogic_vector(WIDTH-1 downto 0);
            a_dly : out std_ulogic_vector(WIDTH-1 downto 0);
            pipeEn : in std_ulogic
        );
    end component;

    -- only fixed-latency operators or MegaWizard independent components are here, rest use the auto-ip feature
	component fneg is
	port (
		a: in std_ulogic_vector(31 downto 0);
		result: out std_ulogic_vector(31 downto 0));
	end component fneg;
	component fnegd is
	port (
		a: in std_ulogic_vector(63 downto 0);
		result: out std_ulogic_vector(63 downto 0));
	end component fnegd;

	component fabs is
	port (
		a: in std_ulogic_vector(31 downto 0);
		result: out std_ulogic_vector(31 downto 0));
	end component fabs;
	component fabsd is
	port (
		a: in std_ulogic_vector(63 downto 0);
		result: out std_ulogic_vector(63 downto 0));
	end component fabsd;
	
	component ftod is
	port (
		clk: in std_ulogic;
		a: in std_ulogic_vector(31 downto 0);
		result: out std_ulogic_vector(63 downto 0));
	end component ftod;
	
	component dtof is
	port (
		clk: in std_ulogic;
		a: in std_ulogic_vector(63 downto 0);
		result: out std_ulogic_vector(31 downto 0));
	end component dtof;

end altera_float;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity latency_delay is
    generic (
        WIDTH : Integer := 1;
        LATENCY : Integer := 1
    );
    port (
        clk: in std_ulogic;
        a: in std_ulogic_vector(WIDTH-1 downto 0);
        a_dly: out std_ulogic_vector(WIDTH-1 downto 0);
        pipeEn: in std_ulogic
    );
end latency_delay;

architecture latency_delay_a of latency_delay is
    type a_dly_arrtype is array (0 to LATENCY) of std_ulogic_vector(WIDTH-1 downto 0);
    signal a_dly_arr: a_dly_arrtype;
begin
    a_dly_arr(0) <= a;
    
    gen_a_dly: for I in 1 to LATENCY generate
        process (clk)
        begin
            if clk'event and clk = '1' then
                if (pipeEn = '1') then
                    a_dly_arr(I) <= a_dly_arr(I-1);
                end if;
            end if;
        end process;
    end generate gen_a_dly;
    
    a_dly <= a_dly_arr(LATENCY);
end latency_delay_a;

library ieee;
use ieee.std_logic_1164.all;

entity fneg is
  port (
    a: in std_ulogic_vector(31 downto 0);
    result: out std_ulogic_vector(31 downto 0));
end fneg;

architecture fneg_a of fneg is
begin
  result(31) <= a(31) xor '1';
  result(30 downto 0) <= a(30 downto 0);
end fneg_a;

library ieee;
use ieee.std_logic_1164.all;

entity fnegd is
  port (
    a: in std_ulogic_vector(63 downto 0);
    result: out std_ulogic_vector(63 downto 0));
end fnegd;

architecture fnegd_a of fnegd is
begin
  result(63) <= a(63) xor '1';
  result(62 downto 0) <= a(62 downto 0);
end fnegd_a;

library ieee;
use ieee.std_logic_1164.all;

entity fabs is
  port (
    a: in std_ulogic_vector(31 downto 0);
    result: out std_ulogic_vector(31 downto 0));
end fabs;

architecture fabs_a of fabs is
begin
  result(31) <= '0';
  result(30 downto 0) <= a(30 downto 0);
end fabs_a;

library ieee;
use ieee.std_logic_1164.all;

entity fabsd is
  port (
    a: in std_ulogic_vector(63 downto 0);
    result: out std_ulogic_vector(63 downto 0));
end fabsd;

architecture fabsd_a of fabsd is
begin
  result(63) <= '0';
  result(62 downto 0) <= a(62 downto 0);
end fabsd_a;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity ftod is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(31 downto 0);
    result: out std_ulogic_vector(63 downto 0));
end ftod;

architecture ftod_a of ftod is
    signal exp_s : std_ulogic_vector(7 downto 0);
    signal exp_d : unsigned(10 downto 0);
    signal zero : std_ulogic;
begin
  exp_s <= a(30 downto 23);             -- extract the exponent
  exp_d <= unsigned("000" & exp_s) + 896;  -- rebias
  zero <= '1' when a = X"00000000" else '0';

  result(63) <= a(31);                  -- sign bit
   -- biased exponent
  result(62 downto 52) <= std_ulogic_vector(exp_d) when zero = '0'
                          else "00000000000";
  result(51 downto 29) <= a(22 downto 0);  -- fraction
  result(28 downto 0) <= "00000000000000000000000000000";
end ftod_a;
  
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity dtof is
  port (
    clk: in std_ulogic;
    a: in std_ulogic_vector(63 downto 0);
    result: out std_ulogic_vector(31 downto 0));
end dtof;

architecture dtof_a of dtof is
    signal exp_s : std_ulogic_vector(7 downto 0);
    signal exp_d : std_ulogic_vector(10 downto 0);
    signal exp : unsigned(10 downto 0);
    signal ovr, inf : std_ulogic;
begin
  exp_d <= a(62 downto 52);             -- extract the exponent
  exp <= unsigned(exp_d) - 896;  -- rebias
  exp_s <= std_ulogic_vector(exp(7 downto 0));
  ovr <= '0' when std_ulogic_vector(exp(10 downto 8)) = "000" else '1';
  inf <= a(62) when ovr = '1' else '0';

  -- sign bit
  result(31) <= a(63);
  -- exponent
  result(30 downto 23) <= exp_s when ovr = '0'
                          else "00000000" when inf = '0'
                          else "11111111";
  -- fraction
  result(22 downto 0) <= a(51 downto 29) when ovr = '0'
                         else "00000000000000000000000";
end dtof_a;

